@@ -38,6 +38,10 @@ _TRACKED_HEADERS = (
     "content-type",
 )
 
+HTTP_VERSION_11 = "HTTP/1.1"
+HTTP_VERSION_10 = "HTTP/1.0"
+DEFAULT_HTTP_VERSION = HTTP_VERSION_10
+
 
 class HTTPConnection(_HTTPConnection):
     pass
@@ -61,28 +65,43 @@ class VerifiedHTTPSConnection(_VerifiedHTTPSConnection):
     def set_ntlm_auth_credentials(cls, username, password):
         cls._ntlm_credentials = get_ntlm_credentials(username, password)
 
+    @classmethod
+    def set_http_version(cls, http_version):
+        if http_version in (HTTP_VERSION_10, HTTP_VERSION_11):
+            cls._http_version = http_version
+        else:
+            logger.debug(
+                "unsupported http-version %r, setting the default %r",
+                http_version,
+                DEFAULT_HTTP_VERSION
+            )
+            cls._http_version = DEFAULT_HTTP_VERSION
+
+    @classmethod
+    def clear_http_version(cls):
+        cls._http_version = None
+        del cls._http_version
+
     @classmethod
     def clear_ntlm_auth_credentials(cls):
         cls._ntlm_credentials = None
         del cls._ntlm_credentials
 
-    def _is_line_blank(self, line):
+    @staticmethod
+    def _is_line_blank(line):
         # for sites which EOF without sending a trailer
-        if not line or line in (b"\r\n", b"\n", b""):
-            last_line_is_blank = True
-        else:
-            last_line_is_blank = False
-        return last_line_is_blank
+        if not line or line in (b"\r\n", b"\n", b"") or not line.strip():
+            return True
+        return False
 
-    def _read_response_line_if_ready(self, response):
+    @staticmethod
+    def _read_response_line_if_ready(response):
         (ready, _, _) = select.select([response.fp], (), (), IO_WAIT_TIMEOUT)
         if ready:
-            line = response.fp.readline()
-        else:
-            line = None
-        return line
+            return response.fp.readline()
 
-    def _flush_response_buffer(self, response):
+    @staticmethod
+    def _flush_response_buffer(response):
         (ready, _, _) = select.select([response.fp], (), (), IO_WAIT_TIMEOUT)
         if ready:
             response.fp.read()
@@ -125,9 +144,18 @@ class VerifiedHTTPSConnection(_VerifiedHTTPSConnection):
             logger.debug("< %r", "{} {} {}".format(version, code, message))
         return version, code, message, response
 
+    def _get_http_version(self):
+        if getattr(self, "_http_version", None):
+            return self._http_version
+        return DEFAULT_HTTP_VERSION
+
     def _get_header_bytes(self, proxy_auth_header=None):
         host, port = self._get_hostport(self._tunnel_host, self._tunnel_port)
-        http_connect_string = "CONNECT {}:{} HTTP/1.0\r\n".format(host, port)
+        http_connect_string = "CONNECT {host}:{port} {http_version}\r\n".format(
+            host=host,
+            port=port,
+            http_version=self._get_http_version()
+        )
         logger.debug("> %r", http_connect_string)
         header_bytes = http_connect_string
         if proxy_auth_header:
@@ -145,13 +173,21 @@ class VerifiedHTTPSConnection(_VerifiedHTTPSConnection):
 
     def _tunnel(self):
         username, password, domain = self._ntlm_credentials
-        logger.debug("attempting to open tunnel using HTTP CONNECT")
-        logger.debug("username: %s, domain: %s", username, domain)
+        logger.debug("* attempting to open tunnel using HTTP CONNECT")
+        logger.debug("* username=%r, domain=%r", username, domain)
+
+        try:
+            workstation = socket.gethostname().upper()
+        except (AttributeError, TypeError, ValueError):
+            workstation = None
+
+        logger.debug("* workstation=%r", workstation)
 
         ntlm_context = HttpNtlmContext(
             username,
             password,
             domain=domain,
+            workstation=workstation,
             auth_type="NTLM",
             ntlm_compatibility=self.ntlm_compatibility,
             ntlm_strict_mode=self.ntlm_strict_mode
@@ -165,38 +201,82 @@ class VerifiedHTTPSConnection(_VerifiedHTTPSConnection):
         if code == PROXY_AUTHENTICATION_REQUIRED:
             authenticate_hdr = None
             match_string = "Proxy-Authenticate: NTLM "
-            last_line_is_blank = False
+            content_length_match_string = "Content-Length:"
+            previous_line = None
+            content_length_header = None
+            body_length = None
             while True:
-                if last_line_is_blank:
-                    line = self._read_response_line_if_ready(response)
-                else:
-                    line = response.fp.readline()
+                line = self._read_response_line_if_ready(response)
                 this_line_is_blank = self._is_line_blank(line)
-                if last_line_is_blank and this_line_is_blank:
-                    self._flush_response_buffer(response)
-                    break
 
-                if this_line_is_blank:
-                    last_line_is_blank = True
+                if body_length is None and this_line_is_blank:
+                    body_length = 0
+                    previous_line = line
                     continue
                 else:
-                    last_line_is_blank = False
+                    if body_length is not None:
+                        body_length += len(line)
+
+                    logger.debug(
+                        "* body length read so far: %s of %s",
+                        body_length,
+                        content_length_header or "unknown"
+                    )
+
+                    if (
+                        body_length is not None
+                        and content_length_header is not None
+                        and body_length >= content_length_header
+                    ):
+                        # we have read the whole response body according to the
+                        # Content-Length response header value.
+                        # We break away from the loop because we have finished draining
+                        # the socket.
+                        break
+                    elif (
+                        this_line_is_blank
+                        and content_length_header is None
+                        and previous_line is not None
+                        and self._is_line_blank(previous_line)
+                    ):
+                        # we have read all the response headers but there was no
+                        # Content-Length header present in the response headers.
+                        # This line is blank and the one before it was blank line too,
+                        # we read the next line (if any) and if it is blank or does not
+                        # exists, we break away from the loop because we have finished
+                        # draining the socket.
+                        line = self._read_response_line_if_ready(response)
+                        if self._is_line_blank(line):
+                            break
 
                 if line.decode("utf-8").startswith(match_string):
+                    # we handle the NTLM challenge message
                     logger.debug("< %r", line)
                     line = line.decode("utf-8")
+                    previous_line = line
                     ntlm_context.set_challenge_from_header(line)
                     authenticate_hdr = ntlm_context.get_authenticate_header()
+                    logger.debug("* authenticate header: %r", authenticate_hdr)
                     continue
+                elif line.decode("utf-8").startswith(content_length_match_string):
+                    # we handle the Content-Length header
+                    logger.info("< %r", line)
+                    line = line.decode("utf-8")
+                    try:
+                        content_length_header = int(
+                            line.replace(content_length_match_string, "").strip()
+                        )
+                    except (ValueError, TypeError):
+                        pass
 
                 if len(line) > _MAXLINE:
                     raise LineTooLong("header line")
 
-                for header in _TRACKED_HEADERS:
-                    if line.decode("utf-8").lower().startswith("{}:".format(header)):
-                        logger.info("< %r", line)
+                logger.debug("< %r", line)
+                previous_line = line
 
             header_bytes = self._get_header_bytes(proxy_auth_header=authenticate_hdr)
+            logger.debug("* sending authenticate header: %r", header_bytes)
             self.send(header_bytes)
             version, code, message, response = self._get_response()
 
