@@ -58,7 +58,7 @@ def get_server_cert(response):
         try:
             server_certificate = socket.getpeercert(True)
         except AttributeError:
-            pass
+            logger.debug("unable to get server certificate")
         else:
             return get_certificate_hash_bytes(server_certificate)
     else:
@@ -100,7 +100,7 @@ def get_auth_type_from_header(header):
     """
     Given a WWW-Authenticate or Proxy-Authenticate header, returns the
     authentication type to use. We prefer NTLM over Negotiate if the server
-    suppports it.
+    supports it.
     """
     if "ntlm" in header.lower():
         return "NTLM"
